﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace HumiditySensor_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
		formCleaner cleaner;
		HumiditySensor humidity = null;
		private ErrorEventBox errorBox;
		private DataGraphBox humidityGraphBox;

		public Form1() {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {
			this.Size = this.MinimumSize;


			errorBox = new ErrorEventBox(this);
			humidityGraphBox = new DataGraphBox(this, "Relative Humidity");
			humidity = new HumiditySensor();

			humidity.Attach += humidity_attach;
			humidity.Detach += humidity_detach;
			humidity.Error += humidity_error;
			humidity.HumidityChange += humidity_change;
			humidity.PropertyChange += humidity_propertychange;

			commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

			try { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
				humidity.Channel = phidgetParameters.Channel; //selects the channel on the device to open
				humidity.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
				humidity.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
				humidity.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

				if (phidgetParameters.isRemote) //are we trying to open a remote device?
				{
					humidity.IsRemote = true;
					Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
					if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
						Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
				} else
					humidity.IsLocal = true;

				humidity.Open(); //open the device specified by the above parameters
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			humidity.Close();
		}

		void humidity_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			HumiditySensor attachedDevice = (HumiditySensor)sender;

			humidityGraphBox.SetTitle(DataGraphBox.GenerateTitle("Humidity", (Phidget)attachedDevice));

			try {
				attachedDevice.DataInterval = attachedDevice.MinDataInterval;

				attachedDevice.HumidityChangeTrigger = attachedDevice.MinHumidityChangeTrigger;

				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.Value = attachedDevice.DataInterval;
				dataIntervalTrk.trkText = attachedDevice.DataInterval.ToString();

				changeTriggerTrk.Unit = "%";
				changeTriggerTrk.minTxt = attachedDevice.MinHumidityChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinHumidityChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxHumidityChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxHumidityChangeTrigger.ToString();
				changeTriggerTrk.Value = 0;
				changeTriggerTrk.trkText = attachedDevice.HumidityChangeTrigger.ToString();
				changeTriggerTrk.trkText = attachedDevice.HumidityChangeTrigger.ToString("F0");

			} catch (PhidgetException ex) { errorBox.addMessage("Error initializing device: " + ex.Message); }

			settingsBox.Visible = true;
			outputBox.Visible = true;
		}

		void humidity_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
		}

		void humidity_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation)
				humidityGraphBox.addGap();
		}

		private void humidity_propertychange(object sender, PropertyChangeEventArgs e)
		{

			switch (e.PropertyName)
			{
				case "DataInterval":
					dataIntervalTrk.Value = humidity.DataInterval;
					break;
				case "HumidityChangeTrigger":
					changeTriggerTrk.Value = humidity.HumidityChangeTrigger;
					break;
				case "DataRate":
					dataIntervalTrk.Value = humidity.DataRate;
					break;
			}
		}

		void humidity_change(object sender, Phidget22.Events.HumiditySensorHumidityChangeEventArgs e) {
			try {
				humTxt.Text = Math.Round(e.Humidity, 4).ToString() + "%";
				humidityGraphBox.addData(e.Humidity, humidity.DataInterval);
			} catch (PhidgetException ex) { errorBox.addMessage("Error reading humidity: " + ex.Message); }
		}

		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					humidity.DataInterval = (int)dataIntervalTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
		}

		private void updateGraphDataInterval()
		{
			try
			{
				if (humidity.HumidityChangeTrigger == 0)
					humidityGraphBox.DataInterval = (1000.0 / humidity.DataRate);
				else
					humidityGraphBox.DataInterval = null;
			}
			catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}

		private void dataIntervalTrk_MouseUp(object sender, MouseEventArgs e) {
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					humidity.DataInterval = (int)Math.Round(dataIntervalTrk.Value);
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					humidity.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data rate: " + ex.Message);
				}
			}
		}

		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					humidity.HumidityChangeTrigger = changeTriggerTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}

		private void changeTriggertrk_MouseUp(object sender, MouseEventArgs e) {
			try {
				humidity.HumidityChangeTrigger = changeTriggerTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}

		private void humidityGraphButton_Click(object sender, EventArgs e) {
			if (!humidityGraphBox.Visible)
				humidityGraphBox.Show();
		}
		private void dataIntervalTrk_SetClick(object sender, EventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					humidity.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					humidity.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
			}

		}

		private void dataIntervalTrk_SetEnter(object sender, KeyEventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				if (e.KeyCode == Keys.Enter)
				{
					try
					{
						humidity.DataInterval = (int)dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					humidity.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
			}
		}

		private void changeTriggerTrk_SetClick(object sender, EventArgs e) {
			try {
				humidity.HumidityChangeTrigger = changeTriggerTrk.Value;
			} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
		}

		private void changeTriggerTrk_SetEnter(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					humidity.HumidityChangeTrigger = changeTriggerTrk.Value;
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}

        private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e)
        {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.isFixedRange = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.Maximum = humidity.MaxDataRate;
					dataIntervalTrk.Minimum = humidity.MinDataRate;
					dataIntervalTrk.maxTxt = humidity.MaxDataRate.ToString();
					dataIntervalTrk.minTxt = humidity.MinDataRate.ToString("F3");
					dataIntervalTrk.trkText = humidity.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = humidity.DataRate;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data rate: " + ex.Message); }

			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					dataIntervalTrk.isIntValue = true;
					dataIntervalTrk.isFixedRange = true;
					dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.Maximum = humidity.MaxDataInterval;
					dataIntervalTrk.Minimum = humidity.MinDataInterval;
					dataIntervalTrk.maxTxt = humidity.MaxDataInterval.ToString();
					dataIntervalTrk.minTxt = humidity.MinDataInterval.ToString();
					dataIntervalTrk.trkText = humidity.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = humidity.DataInterval;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
		}
    }
}
